check.factor.vars<-function(input.data.frame, factor.list) {
# Purpose: checks that the vars in the vector string factor.list are columns of the
#  data matrix input.data.frame - issues a stop if any are not 
#Author: LJT 15/8/06

  #make list of missing columns
  missing<-!(factor.list %in% names(input.data.frame))
  n.missing<-sum(missing)
  #if none missing, return
  if(n.missing==0) return(invisible(NULL))
  #if some missing, stop execution
  if(n.missing==1) {
    text<-paste('The variable',factor.list[missing],'does not exist in the input data file.\n')
  } else {
    text<-paste('The variables',paste(factor.list[missing],collapse=', '),
      'do not exist in the input data file.\n')
  }
  text<-paste(text,'Please check your factor specifications')
  stop(text)
}

write.stats.ddf<-function(stats.file=NULL, append=F, ddf=NULL,ddf.summary=NULL,ddf.gof=NULL) {
#Purpose: Writes selected statistics from the passed in ddf,ddf.summary and ddf.gof objects
# to the file with full path stats.file.  If append=T then current contents not overwritten.
#Author: LJT 16/8/06
#Note - at present doesn't use the ddf.summary and ddf.gof - but will do in future!
#  these could be generated within here, but are passed in as they already exist and they
#  take time to generate

  if(!is.null(stats.file)) sink(file=stats.file,append=append)

  if(!is.null(ddf)) {
    # num params - currently not correct I think, so don't save
    #cat('2010',ddf$par,'\n')
    # AIC
    cat('2020',ddf$criterion,'\n')
    # LNL
    cat('2090',ddf$lnl,'\n')
  }

  if(!is.null(stats.file)) sink()
}

write.stats.dht<-function(stats.file=NULL, append=F, dht=NULL) {
#Purpose: Writes selected statistics from the passed in dht object
# to the file with full path stats.file.  If append=T then current contents not overwritten.
#Author: LJT 16/8/06

  if(!is.null(stats.file)) sink(file=stats.file,append=append)

  if(!is.null(dht)) {
    #true if estimates are available for clusters as well as individuals
    cluster<-!is.null(dht$clusters)
    #number of estimates - global estimate is the last one
    n.ests<-dim(dht$individuals$N)[1]
    #density of clusters
    if(cluster) {
      cat('4010',dht$clusters$D$Estimate[n.ests],'\n')
      cat('4011',dht$clusters$D$cv[n.ests],'\n')
      cat('4012',dht$clusters$D$lcl[n.ests],'\n')
      cat('4013',dht$clusters$D$ucl[n.ests],'\n')
      cat('4014',dht$clusters$D$df[n.ests],'\n')
    }
    #density of individuals
    cat('4020',dht$individuals$D$Estimate[n.ests],'\n')
    cat('4021',dht$individuals$D$cv[n.ests],'\n')
    cat('4022',dht$individuals$D$lcl[n.ests],'\n')
    cat('4023',dht$individuals$D$ucl[n.ests],'\n')
    cat('4024',dht$individuals$D$df[n.ests],'\n')
    #abundance of individuals
    cat('4030',dht$individuals$N$Estimate[n.ests],'\n')
    cat('4031',dht$individuals$N$cv[n.ests],'\n')
    cat('4032',dht$individuals$N$lcl[n.ests],'\n')
    cat('4033',dht$individuals$N$ucl[n.ests],'\n')
    cat('4034',dht$individuals$N$df[n.ests],'\n')
  }

  if(!is.null(stats.file)) sink()
}

create.interval.data<-function(distance,cutpoint,right.trunc) {
#pass in the exact distances as a vector, and another vector containing the cutpoints
#output is a named list with two vectors distbegin and distend
#you can then join these columns to the original ddf.dat data frame
#if a distance is not within the cutpoints, it's given a distance larger than right.trunc
# so it won't be included in the analysis

  n<-length(distance)
  k<-length(cutpoint)
  distbegin<-distend<-numeric(n)

  for(i in 1:n) {

    #check distance is not NA
    if(is.na(distance[i])) {
      #if so, set distbegin and distend to NA
      distbegin[i]<-NA
      distend[i]<-NA
    } else {
      #check distance isn't less than the lowest cutpoint
      # or more than the highest cutpoint
      if((distance[i]<cutpoint[1])|(distance[i]>cutpoint[k])) {
        #if so, set the begin and end values to outside the right truncation point
        distbegin[i]<-right.trunc+1
        distend[i]<-right.trunc+1
      } else {
  
        #find the lower cutpoint
        for (j in 2:k) {
          #go up until you've found a cutpoint where the distance is less
          if(distance[i]<cutpoint[j]) break
        }
        distbegin[i]=cutpoint[j-1]
    
        #find the upper cutpoint
        for (j in seq((k-1),1,by=-1)) {
          #go down until you've found a cutpoint where the distance is more or equal
          if(distance[i]>=cutpoint[j]) break
        }
        distend[i]=cutpoint[j+1]
  
      }
    }
  }
  return(list(distbegin=distbegin,distend=distend))
}

remove.mrds.support.functions<-function() {
#Purpose: Removes the functions declared in this file
# Called at the end of the R scripts in Distance running mrds
  rm(check.factor.vars,pos=1)
  rm(write.stats.ddf,pos=1)
  rm(write.stats.dht,pos=1)
  rm(remove.mrds.support.functions,pos=1)
  rm(create.interval.data,pos=1)
}

